{% extends "!page.html" %}
{% block extrahead %}
    <a href="https://github.com/aliles/begins">
        <img style="position: absolute; top: 0; right: 0; border: 0;"
             src="https://s3.amazonaws.com/github/ribbons/forkme_right_red_aa0000.png"
             alt="Fork me on GitHub">
    </a>
    {{ super() }}
{% endblock %}
